@@ -4,11 +4,12 @@ from neat.indexer import Indexer
 
 
 class NodeGene(object):
-    def __init__(self, ID, node_type, bias=0.0, response=4.924273, activation_type="exp"):
+    def __init__(self, ID, node_type, bias=0.0, response=4.924273, activation_type='sigmoid'):
         """ A node gene encodes the basic artificial neuron model.
             node_type must be 'INPUT', 'HIDDEN', or 'OUTPUT'
         """
         assert activation_type is not None
+        # TODO: Node genes probably shouldn't need to know whether they are input/output/hidden.
         assert node_type in ('INPUT', 'OUTPUT', 'HIDDEN')
 
         self.ID = ID
@@ -18,8 +19,8 @@ class NodeGene(object):
         self.activation_type = activation_type
 
     def __str__(self):
-        return "Node %2d %6s, bias %+2.10s, response %+2.10s" \
-               % (self.ID, self.type, self.bias, self.response)
+        return 'NodeGene(id={0}, type={1}, bias={2}, response={3}, activation={4})'.format(
+            self.ID, self.type, self.bias, self.response, self.activation_type)
 
     def get_child(self, other):
         """ Creates a new NodeGene randomly inheriting attributes from its parents."""
@@ -28,74 +29,32 @@ class NodeGene(object):
         ng = NodeGene(self.ID, self.type,
                       random.choice((self.bias, other.bias)),
                       random.choice((self.response, other.response)),
-                      self.activation_type)
+                      random.choice((self.activation_type, other.activation_type)))
         return ng
 
-    def __mutate_bias(self, config):
+    def mutate_bias(self, config):
         new_bias = self.bias + random.gauss(0, 1) * config.bias_mutation_power
         self.bias = max(config.min_weight, min(config.max_weight, new_bias))
 
-    def __mutate_response(self, config):
+    def mutate_response(self, config):
         """ Mutates the neuron's average firing response. """
         new_response = self.response + random.gauss(0, 1) * config.response_mutation_power
         self.response = max(config.min_weight, min(config.max_weight, new_response))
 
+    def mutate_activation(self, config):
+        self.activation_type = random.choice(config.activation_functions)
+
     def copy(self):
         return NodeGene(self.ID, self.type, self.bias,
                         self.response, self.activation_type)
 
     def mutate(self, config):
         if random.random() < config.prob_mutate_bias:
-            self.__mutate_bias(config)
+            self.mutate_bias(config)
         if random.random() < config.prob_mutate_response:
-            self.__mutate_response(config)
-
-
-class CTNodeGene(NodeGene):
-    """ Continuous-time node gene - used in CTRNNs.
-        The main difference here is the addition of
-        a decay rate given by the time constant.
-    """
-
-    def __init__(self, ID, node_type, bias=1.0, response=1.0, activation_type='exp', time_constant=1.0):
-        super(CTNodeGene, self).__init__(ID, node_type, bias, response, activation_type)
-        self.time_constant = time_constant
-
-    def mutate(self, config):
-        super(CTNodeGene, self).mutate(config)
-        # mutating the time constant could bring numerical instability
-        # do it with caution
-        # if random.random() < 0.1:
-        #    self.__mutate_time_constant()
-
-    def __mutate_time_constant(self, config):
-        """ Warning: perturbing the time constant (tau) may result in numerical instability """
-        self.time_constant += random.gauss(1.0, 0.5) * 0.001
-        if self.time_constant > config.max_weight:
-            self.time_constant = config.max_weight
-        elif self.time_constant < config.min_weight:
-            self.time_constant = config.min_weight
-        return self
-
-    def get_child(self, other):
-        """ Creates a new NodeGene ramdonly inheriting its attributes from parents """
-        assert (self.ID == other.ID)
-
-        ng = CTNodeGene(self.ID, self.type,
-                        random.choice((self.bias, other.bias)),
-                        random.choice((self.response, other.response)),
-                        self.activation_type,
-                        random.choice((self.time_constant, other.time_constant)))
-        return ng
-
-    def __str__(self):
-        return "Node %2d %6s, bias %+2.10s, response %+2.10s, activation %s, time constant %+2.5s" \
-               % (self.ID, self.type, self.bias, self.response,
-                  self.activation_type, self.time_constant)
-
-    def copy(self):
-        return CTNodeGene(self.ID, self.type, self.bias,
-                          self.response, self.activation_type, self.time_constant)
+            self.mutate_response(config)
+        if random.random() < config.prob_mutate_activation:
+            self.mutate_activation(config)
 
 
 class ConnectionGene(object):
@@ -138,12 +97,8 @@ class ConnectionGene(object):
         self.enabled = True
 
     def __str__(self):
-        s = "In %2d, Out %2d, Weight %+3.5f, " % (self.in_node_id, self.out_node_id, self.weight)
-        if self.enabled:
-            s += "Enabled, "
-        else:
-            s += "Disabled, "
-        return s + "Innov %d" % (self.__innov_number,)
+        return 'ConnectionGene(in={0}, out={1}, weight={2}, enabled={3}, innov={4})'.format(
+            self.in_node_id, self.out_node_id, self.weight, self.enabled, self.__innov_number)
 
     def __lt__(self, other):
         return self.__innov_number < other.__innov_number
